import { HttpErrorResponse } from '@angular/common/http';
import { ChangeDetectorRef, Component, ElementRef, Input, OnInit, ViewChild } from '@angular/core';
import { Router } from '@angular/router';
import * as _ from 'lodash';
import { catchError, first } from 'rxjs/operators';
import { Employer } from '../../models/employer.model';
import { Job } from '../../models/job.model';
import { User } from '../../models/user.model';
import { AuthService } from '../../services/auth.service';
import { NotificationService } from '../../services/notification.service';
import { ButtonType } from '../../shared/components/button/buttonType.enum';
import { CustomRegExp } from '../../utils/CustomRegExp';
import { ProfileService } from '../services/profile.service';

enum tabsEnum {
  details,
  credentials,
  employer,
  description,
  avatar,
}

enum showPasswordEnum {
  oldPassword,
  newPassword,
  newPasswordConfirm,
}
@Component({
  selector: 'app-edit',
  templateUrl: './edit.component.html',
  styleUrls: ['./edit.component.scss'],
})
export class EditComponent implements OnInit {
  public tabsEnum = tabsEnum;
  public buttonTypeEnum = ButtonType;
  public currentTab: tabsEnum = tabsEnum.details;

  @Input() userProfile: User;
  public initialUserProfile: User;
  private emailModal = false;
  private passwordModal = false;
  public deleteAccountModal = false;
  private newEmail = '';
  private newEmailConfirm = '';
  private oldPassword = '';
  private newPassword = '';
  private newPasswordConfirm = '';
  private isShowPassword = {
    oldPassword: false,
    newPassword: false,
    newPasswordConfirm: false,
  };
  public jobs: Job[];
  private newJob: Job;
  private selectedJob: Job;
  public employers: Employer[];
  private selectedEmployer: Employer;
  private isAlreadySearching = false;
  public isNewUser = false;
  public isNewEmployer: boolean;
  @ViewChild('searchEmployer') searchEmployer: ElementRef;
  @ViewChild('newJobInput') newJobInput: ElementRef;

  constructor(
    private profileService: ProfileService,
    private notificationService: NotificationService,
    private cdr: ChangeDetectorRef,
    private authService: AuthService,
    private router: Router
  ) {}

  ngOnInit(): void {
    if (history.state.data === 'description') {
      this.currentTab = tabsEnum.description;
    }

    this.profileService.getProfile().then((profile) => {
      this.userProfile = new User(profile);
      this.initialUserProfile = new User({ ...profile });
      this.selectedEmployer = { ...profile.employer };
      if (!profile.employer || !profile.job) {
        this.isNewUser = true;
      }
      const otherJob = new Job({ name: 'Autre' });
      this.profileService.getJobs().subscribe((jobs) => {
        this.jobs = [...jobs, otherJob];

        // Select "Autre" job and set the job's name
        if (jobs.some((job) => job.name === profile.job?.name)) {
          this.selectedJob = { ...profile.job };
        } else {
          this.selectedJob = otherJob;
          this.newJob = profile.job;
        }
      });
    });
  }

  public phoneValid(): boolean {
    return !!this.userProfile.phone.match(CustomRegExp.PHONE);
  }
  public nameValid(): boolean {
    return this.userProfile.name !== '';
  }
  public surnameValid(): boolean {
    return this.userProfile.surname !== '';
  }
  public descriptionValid(): boolean {
    return this.userProfile.description?.length <= 500;
  }
  public emailValid(email: string): boolean {
    return !!email.match(CustomRegExp.EMAIL);
  }
  public passwordValid(password: string): boolean {
    return !!(
      password.match(CustomRegExp.SPECHAR) &&
      password.match(CustomRegExp.DIGIT) &&
      password.match(CustomRegExp.UPPERCASE) &&
      password.match(CustomRegExp.LOWERCASE)
    );
  }

  public navigateTo(tab: tabsEnum): void {
    this.currentTab = tab;
    if (tab === tabsEnum.employer) {
      this.cdr.detectChanges();
      this.selectEmployer(this.userProfile.employer);
      if (this.newJob) this.newJobInput.nativeElement.value = this.userProfile.job.name;
    }
  }

  public cancel(): void {
    this.userProfile = { ...this.initialUserProfile };
  }

  public profileChanged(): boolean {
    return !_.isEqual(this.userProfile, this.initialUserProfile);
  }

  public showEmailModal(): void {
    this.emailModal = true;
  }
  public showPasswordModal(): void {
    this.passwordModal = true;
  }
  public showDeleteAccountModal(): void {
    this.deleteAccountModal = true;
  }
  public showPassword(key: showPasswordEnum): void {
    this.isShowPassword[key] = !this.isShowPassword[key];
  }
  public closeModal(): void {
    this.emailModal = false;
    this.passwordModal = false;
    this.deleteAccountModal = false;
  }

  public isPageValid(): boolean {
    if (this.currentTab === tabsEnum.details) {
      return this.profileChanged() && this.phoneValid() && this.nameValid() && this.surnameValid();
    } else if (this.currentTab === tabsEnum.credentials) {
      if (this.emailModal) {
        return this.emailValid(this.newEmail) && this.newEmail === this.newEmailConfirm;
      } else if (this.passwordModal) {
        return this.passwordValid(this.newPassword) && this.newPassword == this.newPasswordConfirm;
      }
    } else if (this.currentTab === tabsEnum.employer) {
      if (this.isNewUser) {
        if (!this.selectedEmployer || !this.selectedJob) {
          return false;
        }
        return true;
      } else if (
        this.selectedEmployer?.name !== this.userProfile.employer?.name ||
        this.selectedJob?.name !== this.userProfile.job?.name
      ) {
        return true;
      }
      return false;
    } else if (this.currentTab === tabsEnum.description) {
      return this.descriptionValid() && this.initialUserProfile.description !== this.userProfile.description;
    }
  }

  public confirm(): void {
    if (this.currentTab === tabsEnum.details) {
      this.confirmDetails();
    } else if (this.currentTab === tabsEnum.credentials) {
      if (this.emailModal) {
        this.confirmNewEmail();
      } else if (this.passwordModal) {
        this.confirmNewPassword();
      }
    } else if (this.currentTab === tabsEnum.employer) {
      this.confirmEmployer();
    } else if (this.currentTab === tabsEnum.description) {
      this.confirmDescription();
    }
  }

  public confirmDetails(): void {
    this.profileService
      .updateDetails({
        name: this.userProfile.name,
        surname: this.userProfile.surname,
        phone: this.userProfile.phone,
      })
      .subscribe((user: User) => {
        this.notificationService.showSuccess('Vos informations ont bien été enregistrées');
        // Update localStorage
        const updatedUser = {
          ...this.authService.userSubject.value,
          name: user.name,
          surname: user.surname,
          phone: user.phone,
        };
        localStorage.setItem('user', JSON.stringify(updatedUser));
        this.authService.userSubject.next(updatedUser);
        this.updateInitialProfile();
        this.router.navigate(['/profile']);
      });
  }

  public confirmEmployer(): void {
    if (this.newJob) {
      this.selectedJob = this.newJob;
      this.profileService.createJob(this.selectedJob).subscribe((res) => {
        console.log(res);
      });
    }
    if (this.isNewEmployer) {
      this.profileService.createEmployer(this.selectedEmployer).subscribe((res) => {
        console.log(res);
      });
    }

    this.profileService.updateProfile(this.selectedEmployer.name, this.selectedJob.name).subscribe({
      next: () => {
        this.router.navigate(['/profile']);
        this.notificationService.showSuccess('Vos informations ont bien été enregistrées');
      },
      error: () => {
        this.notificationService.showError("Vos informations n'ont pas été enregistrées", 'Une erreur est survenue');
      },
    });
  }

  public confirmDescription(): void {
    this.profileService.updateDescription(this.userProfile.description).subscribe(() => {
      this.notificationService.showSuccess('Vos informations ont bien été enregistrées');
      this.updateInitialProfile();
      this.router.navigate(['/profile']);
    });
  }

  // Updates initialProfile so the button "Valider" is disabled after the user sends new values
  private updateInitialProfile() {
    this.initialUserProfile = { ...this.userProfile };
  }

  public confirmNewEmail(): void {
    if (this.emailValid(this.newEmail) && this.newEmail === this.newEmailConfirm) {
      this.profileService.changeEmail(this.newEmail, this.userProfile.email).subscribe(() => {
        this.closeModal();
        this.notificationService.showSuccess('Veuillez confirmer votre nouvelle adresse grâce au mail envoyé', '');
      });
    }
  }

  public confirmNewPassword(): void {
    if (this.passwordValid(this.newPassword) && this.newPassword == this.newPasswordConfirm) {
      this.profileService
        .changePassword(this.newPassword, this.oldPassword)
        .pipe(
          catchError(async (response: HttpErrorResponse) => {
            if (response.error.statusCode == 401) {
              this.notificationService.showError('Une erreur est survenue', '');
              throw new Error('Une erreur est survenue');
            }
          })
        )
        .subscribe(() => {
          this.notificationService.showSuccess('Votre mot de passe a bien été modifié', '');
          this.closeModal();
        });
      this.oldPassword = '';
      this.isShowPassword.oldPassword = false;
    }
  }

  public confirmDeleteAccount(): void {
    this.authService
      .login(this.userProfile.email, this.oldPassword)
      .pipe(first())
      .subscribe(
        () => {
          this.profileService.deleteProfile().subscribe(() => {
            this.notificationService.showSuccess('Votre compte a bien été supprimé', '');
            this.closeModal();
            this.authService.logout();
          });
        },
        () => {
          this.notificationService.showError('Une erreur est survenue', '');
        }
      );
  }

  // Jobs
  public selectJob(job: Job): void {
    this.selectedJob = job;
    this.newJob = null;
  }
  public isSelectedJob(job: Job): boolean {
    return this.selectedJob && this.selectedJob.name === job.name;
  }
  public isUnexistingJob(): boolean {
    return this.selectedJob && this.selectedJob.name === 'Autre';
  }
  public createNewJob(value: string): void {
    this.newJob = new Job({ name: value, validated: false, hasPersonalOffer: true });
  }

  // Structures
  public onSearchChange(searchString: string): void {
    if (searchString.length <= 2) this.getEmployers();
    this.getEmployers(searchString);
    this.selectedEmployer = new Employer({ name: searchString, validated: false });
    if (!this.employers.map((employer) => employer.name).includes(this.selectedEmployer?.name)) {
      this.isNewEmployer = true;
    } else {
      this.isNewEmployer = false;
    }
  }

  public selectEmployer(employer: Employer): void {
    if (employer) this.searchEmployer.nativeElement.value = employer.name;
    this.selectedEmployer = employer;
    this.employers = [];
    this.isNewEmployer = false;
  }

  private getEmployers(searchString: string = '') {
    if (!this.isAlreadySearching) {
      this.isAlreadySearching = true;
      this.profileService.getEmployers(searchString).subscribe((employers) => {
        this.employers = employers;
        this.isAlreadySearching = false;
      });
    }
  }
}
