import { HttpClient } from '@angular/common/http';
import { Injectable } from '@angular/core';
import * as _ from 'lodash';
import { Observable } from 'rxjs';
import { map } from 'rxjs/operators';
import { Structure } from '../models/structure.model';
import { StructureWithOwners } from '../models/structureWithOwners.model';
import { TempUser } from '../models/temp-user.model';
import { User } from '../models/user.model';
import { Filter } from '../structure-list/models/filter.model';

@Injectable({
  providedIn: 'root',
})
export class StructureService {
  private readonly baseUrl = 'api/structures';
  constructor(private http: HttpClient) {}

  public updateStructureAfterOwnerVerify(idStructure: string, user: User): Observable<any> {
    const emailUser = user.email;
    return this.http
      .put(`${this.baseUrl}/updateAfterOwnerVerify/${idStructure}`, { emailUser })
      .pipe(map((item: Structure) => new Structure(item)));
  }
  public createStructure(structure: Structure, profile: User): Observable<Structure> {
    const userEmail = profile.email;
    return this.http
      .post(`${this.baseUrl}`, { structure, userEmail })
      .pipe(map((item: Structure) => new Structure(item)));
  }

  public editStructure(structure: Partial<Structure>, structureId?: string): Observable<Structure> {
    structure.updatedAt = new Date().toString();
    if (!structureId) structureId = structure._id;
    delete structure._id; // id should not be provided for update
    return this.http
      .patch(`${this.baseUrl}/${structureId}`, structure)
      .pipe(map((item: Structure) => new Structure(item)));
  }

  public isClaimed(id: string, profile: User): Observable<boolean> {
    return this.http.post<boolean>(`${this.baseUrl}/${id}/isClaimed`, profile);
  }

  public claimStructureWithAccount(id: string, email: string): Observable<string[]> {
    return this.http.post<any>(`${this.baseUrl}/${id}/claim`, { email });
  }

  public getStructure(id: string): Observable<Structure> {
    return this.http.get<Structure>(`${this.baseUrl}/${id}`);
  }

  public delete(id: string): Observable<Structure> {
    return this.http.delete<Structure>(`${this.baseUrl}/${id}`);
  }

  public cancelDelete(id: string): Observable<Structure> {
    return this.http.post<Structure>(`${this.baseUrl}/${id}/cancelDelete`, null);
  }

  public removeTempUserFromStructure(idOwner: string, idStructure: string): Observable<any> {
    return this.http.delete<any>(`${this.baseUrl}/${idStructure}/tempUser/${idOwner}`);
  }
  public removeOwnerFromStructure(idOwner: string, idStructure: string): Observable<any> {
    return this.http.delete<any>(`${this.baseUrl}/${idStructure}/owner/${idOwner}`);
  }
  public addOwnerToStructure(user: TempUser, idStructure: string): Observable<string[] | TempUser> {
    return this.http.post<any>(`${this.baseUrl}/${idStructure}/addOwner`, user);
  }

  public getStructuresByName(filters: Filter[]): Observable<Structure[]> {
    return this.getStructures(filters, 'searchByName');
  }

  public getStructures(filters: Filter[], searchUrl: string = 'search', limit?: number): Observable<Structure[]> {
    if (filters && filters.length > 0) {
      let requestUrl = `${this.baseUrl}/${searchUrl}`;
      const queryString = _.find(filters, { name: 'query' });
      if (queryString) {
        _.remove(filters, { name: 'query' });
        requestUrl += `?query=${queryString.value}`;
      }
      const formatedFilters = this.formatFilters(filters);
      return this.http
        .post(requestUrl, { filters: formatedFilters, limit: limit })
        .pipe(map((data: any[]) => data.map((item) => new Structure(item))));
    } else {
      return this.http.get(`${this.baseUrl}`).pipe(map((data: any[]) => data.map((item) => new Structure(item))));
    }
  }

  private formatFilters(filters: Filter[]): object {
    return filters.map((filter) => {
      return { [filter.name]: filter.value };
    });
  }

  public getStructureWithOwners(structureId: string, profile: User): Observable<StructureWithOwners> {
    return this.http.post<any>(`${this.baseUrl}/${structureId}/withOwners`, { emailUser: profile?.email });
  }

  public getTempUsers(structureId: string): Observable<TempUser[]> {
    return this.http.get<TempUser[]>(`${this.baseUrl}/${structureId}/tempUsers`);
  }

  public sendMailOnStructureError(structureId: string, content: string): Observable<any> {
    return this.http.post<any>(`${this.baseUrl}/reportStructureError`, {
      structureId,
      content,
    });
  }
}
